# setup for Monte-Carlo runs with pdfs definition (formerly MC_WQ_pdf_pre.R)
# author: J.A. Torres-Matallana
# organization: Luxembourg Institute of Science and technology (LIST)
#               Wagenigen University and Research Centre (WUR)   
# date: 22.04.2016 - 26.09.2016


setGeneric("MC.setup", function(x) standardGeneric("MC.setup"))

setMethod("MC.setup", signature = "setup", 
          
          function(x){
            # library(EmiStatR)
            # data("Esch_Sure2010")
            # id       = "MC-WQ_3pdf(CODs-NH4s-CODr)_Esch-Sure2010"
            # nsim     = 15
            # seed     = 0.7010607
            # ts.input = Esch_Sure2010
            # mcCores  = 4
            # rng      = rng <- list(
            #   qs   = 150,    # [l/PE/d]
            #   CODs = c(pdf = "nor", mu = 4.379, sigma = 0.73),    # [g/PE/d]
            #   NH4s = c(pdf = "nor", mu = 1.473, sigma = 0.417),    # [g/PE/d]
            #   qf   = 0.05,     # [l/s/ha]
            #   CODf = 0,              # [g/PE/d]
            #   NH4f = 0,              # [g/PE/d]
            #   CODr = c(pdf = "nor", mu = 3.970, sigma = 0.905),     # [mg/l]
            #   NH4r = 0,              # [mg/l]
            #   tf   = 20,          # [min]
            #   nameCSO = "E4",        # [-]
            #   id      = 4,           # [-]
            #   ns      = "FBH Goesdorf",  # [-]
            #   nm      = "GOesdorf",  # [-]
            #   nc      = "Obersauer", # [-]
            #   numc    = 1,          # [-]
            #   use     = "R/I",  # [-]
            #   Ages    = 16.5,        # [ha]
            #   Ared = c(pdf = "nor", mu = 2.036, sigma = 0.083),  # [ha]
            #   tfS     = 10, # [min]
            #   pe      = c(pdf = "uni", min = 550, max = 752), # [PE]
            #   Qd      = 9,   # [l/s]
            #   V       = 190)  # [m3]
            # ar.model  = ar.model <- list(
            #   CODs = .9,
            #   NH4s = .5,
            #   CODr = .1)
            # var.model = var.model <- list(
            #   inp  = c("", ""), # c("CODs", "NH4s"),
            #   w    = c(1.165, 0.189),
            #   A    = matrix(c(0.832, -0.007, -0.131, 0.957), nrow=2, ncol=2),
            #   C    = matrix(c(0.571, -0.001, -0.001, 0.017), nrow=2, ncol=2))
            # folderOutput = "/home/atorres/Documents/02_working/3-Production/03_II-Year/05_models/06_uncertainty/Uncert-opt-comp/output"
            
            id           <- slot(x, "id")
            nsim         <- slot(x, "nsim")
            seed         <- slot(x, "seed")
            mcCores      <- slot(x, "mcCores")
            ts.input     <- slot(x, "ts.input")
            rng          <- slot(x, "rng")
            ar.model     <- slot(x, "ar.model")
            var.model    <- slot(x, "var.model")
            folderOutput <- slot(x, "folderOutput")
            
            #=============================================================================================
            # MC simulations and set-up
            #=============================================================================================
            par <- rng
            
            # generation of random numbers (uniform distribution)
            set.seed(seed)
            # i <-1
            npar <- 0
            
            # par[[18]]["pdf"]=="uni"
            # par[[2]][1]
            
            
            indexVAR <- 0
            l <- 1
            # k <- 1
            ifelse(length(var.model[[1]]) > 0,
                   {
                     if(var.model[["inp"]][1] != "") 
                     {for(k in 1:length(var.model[["inp"]])){
                       indexVAR[l] <- which(names(rng)==var.model[["inp"]][k])
                       l <- l+1
                     }
                       indexVAR
                     }else{indexVAR <- c(0,0)}
                   }, indexVAR <- c(0,0)
            )
            
            # ifelse(var.model[["inp"]][1] != "", 
            #        {for(k in 1:length(var.model[["inp"]])){
            #          indexVAR[l] <- which(names(rng)==var.model[["inp"]][k])
            #          l <- l+1}
            #        }, indexVAR <- c(0,0))
            
            for (i in 1:length(rng)){
              if(i == indexVAR[2]){next}
              # i <- 20
              if(length(rng[[i]]) > 1 & rng[[i]][1] == "uni") {case <- 1}
              # i <- 2
              if(length(rng[[i]]) > 1 & rng[[i]][1] == "nor" & any(names(rng)[[i]] == names(ar.model)) & 
                 !any(names(rng)[[i]] == var.model[["inp"]])) {case <- 2}
              # i <- 18
              if(length(rng[[i]]) > 1 & rng[[i]][1] == "nor" & !any(names(rng)[[i]] == names(ar.model))) {case <- 3}
              # i <- 1
              if(length(rng[[i]]) == 1) {case <- 4}
              # i <- 2
              if(length(rng[[i]]) > 1 & rng[[i]][1] == "nor" & any(names(rng)[[i]] == names(ar.model)) & 
                 any(names(rng)[[i]] == var.model[["inp"]])) {case <- 5}
              
              switch(case,
                     # case 1: uniform values ---------------------------------------------------------------------------------------------------
                     {r <- matrix(NA, nrow=nsim, ncol = nrow(ts.input)) 
                     j <- 1
                     for(j in 1:nsim){
                       r[j,] <- runif(nrow(ts.input), min=as.numeric(rng[[i]]["min"]), max=as.numeric(rng[[i]]["max"]))    
                     }
                     par[[i]] <- r 
                     npar <- npar+1
                     }, 
                     
                     # case 2: normal autocorrelated time series (AR1 model) ----------------------------------------------------------------------
                     {requireNamespace("lmom")                                                                                      
                       r <- matrix(NA, nrow=nsim, ncol = nrow(ts.input))
                       j <- 1
                       for(j in 1:nsim){
                         # r[j,] <- quanor(runif(nrow(ts.input)), c(as.numeric(rng[[i]]["mu"]), as.numeric(rng[[i]]["sigma"])))
                         
                         y1 <- arima.sim(n = nrow(ts.input) , list(order=c(1,0,0), ar=ar.model[[names(rng)[[i]]]]))  # ar =.5
                         m1 <- mean(y1);    s1 <- sd(y1)
                         m2 <- as.numeric(rng[[i]]["mu"]); s2 <- as.numeric(rng[[i]]["sigma"])
                         y2 <- m2 +(y1-m1)*s2/s1
                         
                         r[j,] <- y2
                       }
                       par[[i]] <- r  
                       npar <- npar+1 
                     },
                     
                     # case 3: normal non-autocorrelated time series ----------------------------------------------------------------------------------
                     {requireNamespace("lmom") 
                       r <- matrix(NA, nrow=nsim, ncol = 1)
                       # i <- 7
                       # j <- 1
                       for(j in 1:nsim){
                         r[j,1] <- quanor(runif(1), c(as.numeric(rng[[i]]["mu"]), as.numeric(rng[[i]]["sigma"])))
                       }
                       par[[i]] <- r  
                       npar <- npar+1 
                     },
                     
                     # case 4: constant value ---------------------------------------------------------------------------------------------------------
                     {par[[i]] <- rng[[i]]
                     },
                     
                     # # case 5: normal auto- and cross-correlated time series (var.model) --------------------------------------------------------------
                     # {
                     #   library(lmom)
                     #   library(mAr)
                     #   r <- matrix(NA, nrow=nsim, ncol = nrow(ts.input))
                     #   
                     #   index <- 0
                     #   l <- 1
                     #   for(k in 1:length(var.model[["inp"]])){
                     #     
                     #     for(j in 1:nsim){
                     #       
                     #       y1 <- mAr.sim(w = var.model[["w"]], A = var.model[["A"]], C = var.model[["C"]], N = nrow(ts.input))
                     #       
                     #       index[l] <- which(names(rng)==var.model[["inp"]][k])
                     #       
                     #       r[j,] <- quanor(runif(nrow(ts.input)), c(as.numeric(rng[[index[l]]]["mu"]), as.numeric(rng[[index[l]]]["sigma"])))
                     #       
                     #       m1 <- mean(y1[,l]); s1 <- sd(y1[,l])
                     #       m2 <- mean(r[j,]); s2 <- sd(r[j,])
                     #       y2 <- m2 +(y1[,l]-m1)*s2/s1
                     #       
                     #       r[j,] <- y2
                     #     }
                     #     l <- l+1
                     #     
                     #     par[[index[l-1]]] <- r
                     #     npar <- npar+1
                     #   }
                     # }
                     
                     # case 5: normal auto- and cross-correlated time series (var.model), parallel code -------------------------------
                     {
                       var1 <- (matrix(NA, nrow=1, ncol = nrow(ts.input)) )
                       var2 <- (matrix(NA, nrow=1, ncol = nrow(ts.input)) )
                       
                       obj1 <- 1:nsim
                       
                       requireNamespace("parallel")
                       requireNamespace("doParallel")
                       requireNamespace("foreach")
                       cl <- makeCluster(mcCores, outfile="")
                       registerDoParallel(cl, cores=mcCores)
                       numCores <- detectCores()
                       
                       comb <- function(x, ...) {
                         lapply(seq_along(x),
                                function(i) c(x[[i]], lapply(list(...), function(y) y[[i]])))
                       }
                       
                       rp <- foreach(obj1 = obj1, .packages = c("lmom", "mAr", "parallel", "doParallel", "foreach"), .export=c("obj1"),
                                     .errorhandling = "pass", .verbose=TRUE, .combine = "rbind") %dopar% {
                                       
                                       # j <- 1
                                       j <- obj1
                                       print(paste(j, ", ", mcCores, " of ", numCores, " cores for creating VAR model", sep=""))
                                       
                                       y1 <- mAr.sim(w = var.model[["w"]], A = var.model[["A"]], C = var.model[["C"]], N = nrow(ts.input))
                                       
                                       # code for only bivariate case
                                       m1 <- mean(y1[,1]); s1 <- sd(y1[,1])
                                       m2 <- as.numeric(rng[[indexVAR[1]]]["mu"]); s2 <- as.numeric(rng[[indexVAR[1]]]["sigma"])
                                       y2 <- m2 +(y1[,1]-m1)*s2/s1
                                       var1[1,] <- y2
                                       
                                       m1 <- mean(y1[,2]); s1 <- sd(y1[,2])
                                       m2 <- as.numeric(rng[[indexVAR[2]]]["mu"]); s2 <- as.numeric(rng[[indexVAR[2]]]["sigma"])
                                       y2 <- t(as.data.frame(m2 +(y1[,2]-m1)*s2/s1))
                                       var2[1,] <- y2
                                       
                                       return(c(var1, var2))
                                     }
                       
                       stopCluster(cl)
                       # closeAllConnections()
                       
                       # asignation only for bivariate case:
                       par[[indexVAR[1]]] <- rp[,1:nrow(ts.input)]  
                       par[[indexVAR[2]]] <- rp[,(nrow(ts.input)+1):(nrow(ts.input)*2)]  
                       npar <- npar+2  
                     }
                     
              )
            }
            
            npars <- length(rng)
            
            return(list(id = id, nsim=nsim, seed=seed, mcCores = mcCores, ts.input=ts.input, rng=rng, par=par, 
                        ar.model=ar.model, var.model=var.model, folderOutput=folderOutput))
          }
)